@@ -13,11 +13,12 @@
 Version information is not placed in the top-level Makefile by default
 */
 #ifndef VERSION
-#define VERSION "2.91"
+#define VERSION "2.94"
 #endif
 /*
  *		This file is part of the sysvinit suite,
  *		Copyright (C) 1991-2004 Miquel van Smoorenburg.
+ *		Copyright (C) 2017-2019 Jesse Smith
  *
  *		This program is free software; you can redistribute it and/or modify
  *		it under the terms of the GNU General Public License as published by
@@ -52,7 +53,11 @@ Version information is not placed in the top-level Makefile by default
 #include <string.h>
 #include <signal.h>
 #include <termios.h>
+#ifdef __FreeBSD__
+#include <utmpx.h>
+#else
 #include <utmp.h>
+#endif
 #include <ctype.h>
 #include <stdarg.h>
 #include <sys/ttydefaults.h>
@@ -62,6 +67,9 @@ Version information is not placed in the top-level Makefile by default
 #ifdef WITH_SELINUX
 #  include <selinux/selinux.h>
 #endif
+#ifdef __FreeBSD__
+extern char **environ;
+#endif
 
 #ifdef __i386__
 #  ifdef __GLIBC__
@@ -133,7 +141,7 @@ int wrote_wtmp_reboot = 1;	/* Set when we wrote the reboot record */
 int wrote_utmp_reboot = 1;	/* Set when we wrote the reboot record */
 int wrote_wtmp_rlevel = 1;	/* Set when we wrote the runlevel record */
 int wrote_utmp_rlevel = 1;	/* Set when we wrote the runlevel record */
-int sltime = 5;			/* Sleep time between TERM and KILL */
+int sleep_time = WAIT_BETWEEN_SIGNALS;    /* Sleep time between TERM and KILL */
 char *argv0;			/* First arguments; show up in ps listing */
 int maxproclen;			/* Maximal length of argv[0] with \0 */
 struct utmp utproto;		/* Only used for sizeof(utproto.ut_id) */
@@ -144,7 +152,7 @@ int main(int, char **);
 
 /*	Used by re-exec part */
 int reload = 0;			/* Should we do initialization stuff? */
-char *myname="/sbin/init";	/* What should we exec */
+char *myname=INIT;		/* What should we exec */
 int oops_error;			/* Used by some of the re-exec code. */
 const char *Signature = "12567362";	/* Signature for re-exec fd */
 
@@ -288,7 +296,7 @@ void send_state(int fd)
 	fprintf(fp, "-SI%u\n", got_signals);
 	fprintf(fp, "-WR%d\n", wrote_wtmp_reboot);
 	fprintf(fp, "-WU%d\n", wrote_utmp_reboot);
-	fprintf(fp, "-ST%d\n", sltime);
+	fprintf(fp, "-ST%d\n", sleep_time);
 	fprintf(fp, "-DB%d\n", did_boot);
 
 	for (p = family; p; p = p->next) {
@@ -416,7 +424,7 @@ static CHILD *get_record(FILE *f)
 				}
 				break;
 			case D_SLTIME:
-				if (fscanf(f, "%d\n", &sltime) == EOF && errno != 0) {
+				if (fscanf(f, "%d\n", &sleep_time) == EOF && errno != 0) {
 					fprintf(stderr, "%s (%d): %s\n", __FILE__, __LINE__, strerror(errno));
 				}
 				break;
@@ -521,8 +529,12 @@ int receive_state(int fd)
  *	Set the process title.
  */
 #ifdef __GNUC__
+#ifndef __FreeBSD__
 __attribute__ ((format (printf, 1, 2)))
 #endif
+#endif
+/* This function already exists on FreeBSD. No need to delcare it. */
+#ifndef __FreeBSD__
 static int setproctitle(char *fmt, ...)
 {
 	va_list ap;
@@ -542,6 +554,7 @@ static int setproctitle(char *fmt, ...)
 
 	return len;
 }
+#endif
 
 /*
  *	Set console_dev to a working console.
@@ -832,7 +845,7 @@ void console_stty(void)
 #endif /* IUTF8 */
 		;
 	tty.c_oflag = OPOST|ONLCR;
-	tty.c_lflag = ISIG|ICANON|ECHO|ECHOCTL|ECHOPRT|ECHOKE;
+	tty.c_lflag = ISIG|ICANON|ECHO|ECHOCTL|ECHOE|ECHOKE;
 
 #if defined(SANE_TIO) && (SANE_TIO == 1)
 	/*
@@ -1705,7 +1718,7 @@ void read_inittab(void)
 	/*
 	 *	Yup, but check every second if we still have children.
 	 */
-	for(f = 0; f < sltime; f++) {
+	for(f = 0; f < sleep_time; f++) {
 		for(ch = family; ch; ch = ch->next) {
 			if (!(ch->flags & KILLME)) continue;
 			if ((ch->flags & RUNNING) && !(ch->flags & ZOMBIE))
@@ -1959,7 +1972,7 @@ int read_level(int arg)
 		foo = arg;
 		ok = 1;
 	}
-	if (ok == 2) sltime = st;
+	if (ok == 2) sleep_time = st;
 
 #endif /* INITLVL */
 
@@ -2160,9 +2173,11 @@ int make_pipe(int fd)
 
 /*
  *	Attempt to re-exec.
+ *      Renaming to my_re_exec since re_exec is now a common function name
+ *      which conflicts.
  */
 static
-void re_exec(void)
+void my_re_exec(void)
 {
 	CHILD		*ch;
 	sigset_t	mask, oldset;
@@ -2224,7 +2239,7 @@ void re_exec(void)
 
 	/*
 	 *	We shouldn't be here, something failed. 
-	 *	Bitch, close the state pipe, unblock signals and return.
+	 *	Close the state pipe, unblock signals and return.
 	 */
 	init_freeenv(env);
 	close(fd);
@@ -2279,7 +2294,7 @@ void fifo_new_level(int level)
 		runlevel = read_level(level);
 		if (runlevel == 'U') {
 			runlevel = oldlevel;
-			re_exec();
+			my_re_exec();
 		} else {
 			if (oldlevel != 'S' && runlevel == 'S') console_stty();
 			if (runlevel == '6' || runlevel == '0' ||
@@ -2460,22 +2475,22 @@ void check_init_fifo(void)
 	}
 	switch(request.cmd) {
 		case INIT_CMD_RUNLVL:
-			sltime = request.sleeptime;
+			sleep_time = request.sleeptime;
 			fifo_new_level(request.runlevel);
 			quit = 1;
 			break;
 		case INIT_CMD_POWERFAIL:
-			sltime = request.sleeptime;
+			sleep_time = request.sleeptime;
 			do_power_fail('F');
 			quit = 1;
 			break;
 		case INIT_CMD_POWERFAILNOW:
-			sltime = request.sleeptime;
+			sleep_time = request.sleeptime;
 			do_power_fail('L');
 			quit = 1;
 			break;
 		case INIT_CMD_POWEROK:
-			sltime = request.sleeptime;
+			sleep_time = request.sleeptime;
 			do_power_fail('O');
 			quit = 1;
 			break;
@@ -2681,7 +2696,7 @@ void process_signals()
 #endif
 		if (runlevel == 'U') {
 			runlevel = oldlevel;
-			re_exec();
+			my_re_exec();
 		} else {
 			if (oldlevel != 'S' && runlevel == 'S') console_stty();
 			if (runlevel == '6' || runlevel == '0' ||
@@ -2902,7 +2917,7 @@ int telinit(char *progname, int argc, char **argv)
 
 	while ((f = getopt(argc, argv, "t:e:")) != EOF) switch(f) {
 		case 't':
-			sltime = atoi(optarg);
+			sleep_time = atoi(optarg);
 			break;
 		case 'e':
 			if (env == NULL)
@@ -2935,7 +2950,7 @@ int telinit(char *progname, int argc, char **argv)
 			usage(progname);
 		request.cmd = INIT_CMD_RUNLVL;
 		request.runlevel  = argv[optind][0];
-		request.sleeptime = sltime;
+		request.sleeptime = sleep_time;
 	}
 
 	/* Change to the root directory. */
@@ -2977,7 +2992,7 @@ int telinit(char *progname, int argc, char **argv)
 				progname, INITLVL);
 			exit(1);
 		}
-		fprintf(fp, "%s %d", argv[optind], sltime);
+		fprintf(fp, "%s %d", argv[optind], sleep_time);
 		fclose(fp);
 
 		/* And tell init about the pending runlevel change. */
