@@ -58,8 +58,6 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
-char *Version = "@(#)killall5 2.86 31-Jul-2004 miquels@cistron.nl";
-
 #ifndef PATH_MAX
 #  ifdef MAXPATHLEN
 #    define PATH_MAX MAXPATHLEN
@@ -435,7 +433,11 @@ int check4nfs(const char * path, char * real)
 
 	} while (1);
 
-	if (real) strcpy(real, curr);
+	if (real)     /* real is defined elsewhere as being PATH_MAX + 1 */
+        {
+           memset(real, '\0', PATH_MAX + 1);
+           strncpy(real, curr, PATH_MAX);
+        }
 
 	if (errno == EINVAL) {
 		const size_t nlen = strlen(curr);
@@ -484,6 +486,7 @@ int readproc(int do_stat)
 	unsigned long	startcode, endcode;
 	int		pid, f;
 	ssize_t		len;
+        char            process_status[11];
 
 	/* Open the /proc directory. */
 	if (chdir("/proc") == -1) {
@@ -567,11 +570,20 @@ int readproc(int do_stat)
 
 			/* Get session, startcode, endcode. */
 			startcode = endcode = 0;
+                        /*
 			if (sscanf(q, 	"%*c %*d %*d %d %*d %*d %*u %*u "
 					"%*u %*u %*u %*u %*u %*d %*d "
 					"%*d %*d %*d %*d %*u %*u %*d "
 					"%*u %lu %lu",
 					&p->sid, &startcode, &endcode) != 3) {
+                        */
+                        if (sscanf(q,   "%10s %*d %*d %d %*d %*d %*u %*u "
+                                        "%*u %*u %*u %*u %*u %*d %*d "
+                                        "%*d %*d %*d %*d %*u %*u %*d "
+                                        "%*u %lu %lu",
+                                        process_status, 
+                                        &p->sid, &startcode, &endcode) != 4) {
+
 				p->sid = 0;
 				nsyslog(LOG_ERR, "can't read sid from %s\n",
 					path);
@@ -586,6 +598,18 @@ int readproc(int do_stat)
 			if (startcode == 0 && endcode == 0)
 				p->kernel = 1;
 			fclose(fp);
+                        if ( (strchr(process_status, 'D') != NULL) ||
+                             (strchr(process_status, 'Z') != NULL) ){
+                           /* Ignore zombie processes or processes in
+                              disk sleep, as attempts
+                              to access the stats of these will
+                              sometimes fail. */
+                              if (p->argv0) free(p->argv0);
+                              if (p->argv1) free(p->argv1);
+                              if (p->statname) free(p->statname);
+                             free(p);
+                             continue;
+                        }
 		} else {
 			/* Process disappeared.. */
 			if (p->argv0) free(p->argv0);
@@ -923,13 +947,14 @@ void usage(void)
 void pidof_usage(void)
 {
    printf("pidof usage: [options] <program-name>\n\n");
-   printf(" -c         Return PIDs with the same root directory\n");
-   printf(" -h         Display this help text\n");
-   printf(" -n         Avoid using stat system function on network shares\n");
-   printf(" -o <pid>   Omit results with a given PID\n");
-   printf(" -q         Quiet mode. Do not display output\n");
-   printf(" -s         Only return one PID\n");
-   printf(" -x         Return PIDs of shells running scritps with a matchign name\n");
+   printf(" -c           Return PIDs with the same root directory\n");
+   printf(" -h           Display this help text\n");
+   printf(" -f <format>  Display PIDs in a given printf-style format\n");
+   printf(" -n           Avoid using stat system function on network shares\n");
+   printf(" -o <pid>     Omit results with a given PID\n");
+   printf(" -q           Quiet mode. Do not display output\n");
+   printf(" -s           Only return one PID\n");
+   printf(" -x           Return PIDs of shells running scritps with a matchign name\n");
    printf("\n");
 }
 
@@ -974,6 +999,7 @@ int main_pidof(int argc, char **argv)
 	int		chroot_check = 0;
 	struct stat	st;
 	char		tmp[512];
+        char            *format = NULL;
 
 	omit = (OMIT*)0;
 	nlist = (NFS*)0;
@@ -982,7 +1008,7 @@ int main_pidof(int argc, char **argv)
 	if ((token = getenv("PIDOF_NETFS")) && (strcmp(token,"no") != 0))
 		flags |= PIDOF_NETFS;
 
-	while ((opt = getopt(argc,argv,"qhco:sxn")) != EOF) switch (opt) {
+	while ((opt = getopt(argc,argv,"qhco:f:sxn")) != EOF) switch (opt) {
 		case '?':
 			nsyslog(LOG_ERR,"invalid options on command line!\n");
 			closelog();
@@ -993,6 +1019,9 @@ int main_pidof(int argc, char **argv)
                 case 'h':
                         pidof_usage();
                         exit(0);
+                case 'f':
+                        format = optarg;
+                        break;
 		case 'o':
 			here = optarg;
 			while ((token = strsep(&here, ",;:"))) {
@@ -1089,9 +1118,14 @@ int main_pidof(int argc, char **argv)
 				}
 
 				if ( ~flags & PIDOF_QUIET ) {
-					if (!first)
+                                    if (format)
+                                       printf(format, p->pid);
+                                    else
+                                    {
+					if (! first)
 						printf(" ");
 					printf("%d", p->pid);
+                                    }
 				}
 				first = 0;
 			}
